#!/usr/bin/env python
"""
Cross-platform handling of Gnuplot. The script accepts the
same command-line arguments as on Unix systems, but on
Windows the scriptfile (if present) is renamed to GNUPLOT.INI
and gnuplot is run without arguments.
"""

import sys, os

if len(sys.argv) < 2:
    print 'Error: not enough arguments!'; sys.exit(1)

if sys.platform.startswith('win'):
    # parse the command-line; it's easy since we drop all X11 options and
    # only need to grab the scriptfile name, and it's the last argument:
    scriptfile = sys.argv[-1]
    if os.path.isfile('GNUPLOT.INI'):
        os.remove('GNUPLOT.INI')
    os.rename(scriptfile, 'GNUPLOT.INI')
    failure = os.system('wgnuplot')  # Gnuplot 4.0
    if failure:
        print 'Could not find Gnuplot on Windows.\n'\
              'The name of the file is wgnupl32.exe. Either you have\n'\
              'not installed Gnuplot or the path to the .exe file is\n'\
              'not registered in your %PATH% environment variable.'
        sys.exit(1)
elif os.name == 'posix':
    os.system('gnuplot ' + ' '.join(sys.argv[1:]))
else:
    print 'Platform', sys.platform, ' on OS', os.name, 'is not supported'
    sys.exit(1)
