"""
%(easyviz)s is a light-weight interface to curve plotting in Python scripts.
The easyplot interface was designed with three ideas in mind:

1. The primary goal of Easyplot is to plot array data with a minimal
set of keystrokes using a Matlab-syntax. A simple 

plot(x,y) 

plots the data in (the NumPy array) y versus the data in (the NumPy
array) x. If you need legends, control of the axis, as well as an
additional curve, this is obtained by the standard Matlab-style commands:

legend('first curve')
axis(0, 10, -1, 1)
hold('on')
plot(x, y2, 'ro--')  # ro-- gives red o markers with dashes -- (?????)
legend('second curve')
title('My first easyplot demo')
hardcopy('tmp.ps')   # save figure in file tmp.ps

One may also use a more convenient compound plot command to achieve
the same result:

plot(x, y, '', x, y2, 'ro--', legend=('first curve',
     'second curve'), title='My first easyplot demo',
     hardcopy='tmp.ps')

2. Easyplot is just an interface to other plotting packages that can
be called from Python. Such plotting packages are referred to as
backends. Several backends are supported: Gnuplot, Matplotlib,
Pmw.Blt.Graph, PyX. Hence, scripts that use Easyplot commands only
can work with a variety of backends, depending on what you have
installed on the machine in question and what quality of the plots
you demand. For example, swiching from Gnuplot to Matplotlib is trivial.
Scripts with Easyplot commands will most probably run anywhere since
at least the Gnuplot package can always be installed right away on a
platform.

3. The Easyplot interface is minimalistic, aimed at rapid prototyping
of plots. This makes the easyplot code easy to read and extend (e.g.,
with new backends). If you need more sophisticated plotting, like
controlling tickmarks, inserting annotations, etc., you must grab the
backend object and use the backend-specific syntax to fine-tune the
plot. The idea is that you can get away with Easyplot and a plotting
package-independent script "95%" of the time - only now and then there
will be demand for package-dependent code for fine-tuning.


"""

__author__ = "Rolv Erlend Bredesen, Hans Petter Langtangen, Johannes Hofaker Ring"
__version__ = "0.1"

# from scitools.easyplot import * is the same as from scitools.easyplot.gnuplot_
from common import *
backend = 'gnuplot_'
#backend = 'pyx_'
#backend = 'blt_'
exec('from %s  import *' % backend)

# add plot doc string to package doc string:
__doc__ += plot.__doc__

def get_backend():
    """
    Return the current backend object (used for direct access
    to the underlying plotting package when there is need for
    advanced plotting beyond the plain easyplot functionality).
    """
    return plt._g


