import pickle
import os
import operator 


class Line(object):
    """ A way to store lines used for plotting"""
    # According to easyplot.common.PlotBackend
    _colors = "b g r m c y k".split()
    _markers = "o + x * s d v ^ < > p h .".split()
    _linestyles = "- : -. --".split()
    _sizes = "1 2 3 4 5 6 7 8 9".split()
    _styledoc = {'y': 'yellow',
                 'm': 'magenta',
                 'c': 'cyan',          
                 'r': 'red',           
                 'g': 'green',         
                 'b': 'blue',     
                 'w': 'white',    
                 'k': 'black',

                 '.': 'point',       
                 'o': 'circle',      
                 'x': 'x-mark',       
                 '+': 'plus',              
                 '*': 'star',
                 's': 'square',
                 'd': 'diamond',
                 'v': 'triangle (down)',
                 '^': 'triangle (up)',
                 '<': 'triangle (left)',
                 '>': 'triangle (right)',
                 'p': 'pentagram',
                 'h': 'hexagram',
                 
                 '-': 'solid',
                 ':': 'dotted',
                 '-.':'dashdot',
                 '--':'dashed'}
        
    def __init__(self, *args, **kwargs):
        # define line attributes:
        self.attrs = {
            'legend':    "",
            'color':     "", 
            'line_type': "", 
            'marker':    "", 
            'linewidth': "",
            'pointsize': "",         
            'x':         None,
            'y':         None
            }

        self.set(**kwargs)

    def setformat(self, format):
        """
        Set the correct format from the given format string.
        Erroneous chars will be ignored.
        When there are multiple format characters for a property,
        the last one will count.
        """
        if isinstance(format,str) and len(format) > 0:
            color = ""
            line_type = ""
            marker = ""
            linewidth = ""
            pointsize = ""
            # Notice that '--' is after '-' and '-.' in the _linestyle alphabet
            for item in self._linestyles:
                if item in format: line_type=item
            for item in format:
                if item in self._colors:
                    color = item
                elif item in self._markers:
                    if  item == '.':
                        if ('.' in line_type) and ('.'.count(format) == 1):
                            pass
                        else:
                            marker = item # same as '.'
                    else:
                        marker = item 
                elif item in self._sizes:
                    # Should this int describe pointsize or linewidth
                    print item
                    self.attrs['pointsize'] = item
                    self.attrs['linewidth'] = item
                        
            if (color in self._colors) or (color == ""):
                self.attrs['color'] = color
            else:
                print "Illegal color choice, %s is not known" %(color)
            if ((line_type != "") or (marker != "")):
                if line_type in self._linestyles:
                    self.attrs['line_type'] = line_type
                elif line_type == "":
                    self.attrs['line_type'] = line_type # Since marker is known
                else:
                    print "Illegal linetype choice, \
                    %s is not known" %(line_type)
                if marker in self._markers:
                    self.attrs['marker'] = marker
                elif marker == "":
                    self.attrs['marker'] = marker # Since linestyle is known
                else:
                    print "Illegal marker choice, %s is not known" %(marker)

    def set(self, **kwargs):
        """
        Set line properties. Legal keyword arguments: x, y, format.
        The format string is passed on to setformat
        """

        # Here x,y values can be any SequenceType
        # The proper casting should be in the backends plotroutine

        if 'y' in kwargs:
            if not operator.isSequenceType(kwargs['y']):
                raise TypeError, "Can only plot sequence types"
            self.attrs['y'] = kwargs['y']
            if 'format' in kwargs:
                self.setformat(kwargs['format'])
            if 'x' in kwargs:  # will only set x variable if y is set
                if kwargs['x'] == 'auto':  # now x is the indicies of y 
                    self.attrs['x'] = range(len(self.attrs['y']))
                else:
                    if not operator.isSequenceType(kwargs['x']):
                        raise TypeError, "Can only plot sequence types"
                    self.attrs['x'] = kwargs['x']
            # Consitency check
            # Check for same length of x and y

    # Write a get function here

            
class Figure(object):
    """
    The Figure class holds figure attributtes like xlabel, ylabel, lines...
    Surface instances are coming in next level of conformance.
    """
   
    def __init__(self, *args, **kwargs):
        self.attrs = {
            'xlabel': "x",
            'ylabel': "y",
            'title': "",
            'scale': "linear",
            #'hardcopy': False,
            'hold': 'off',
            # User specified axes ranges 
            'xmin': None,
            'xmax': None,
            'ymin': None,
            'ymax': None,
            'zmin': None,
            'zmax': None,
            # 'default_filename': "tmp.eps", # See BaseClass.hardcopy instead
            # 'filename': None, # User specified filename
            # 'fixed_view':   False #Same as the movie variable
            }
        # Need some check for legal attributes
        # Use a Copy of 'BaseClass.set' and 'get' here
        # self.lines=[]
        # self.lines.append(Line())
        
        try: self.lines
        except: self.lines = []

                             
    def set(self, *args, **kwargs):
        """
        """
        # Can steel code from VizCurve here,
        # If movie=='on' there should be a search for axis range
        # Set hold state
        if 'hold' in kwargs:
            tmparg = kwargs['hold']
            if not tmparg in ('on', 'off'):
                if tmparg or tmparg == None: tmparg = 'on'
                else: tmparg = 'off'
            self.attrs['hold'] = tmparg
            
        if 'log' in kwargs:
            if kwargs['log'] == 'x':
                self.attrs['scale'] = 'logx'
            elif kwargs['log'] == 'y':
                self.attrs['scale'] = 'logy'
            elif kwargs['log'] == 'xy':
                self.attrs['scale'] = 'loglog'
            # Note: The only way to reset scale to linear after log is to use
            # plot(args,log=None)
            elif kwargs['log'] == None:
                self.attrs['scale'] = 'linear'
            
        # Set xlabel
        if 'xlabel' in kwargs:
            try:
                tmparg = str(kwargs['xlabel'])
                self.attrs['xlabel'] = tmparg
            except: raise Exception
            
        # Set ylabel
        if 'ylabel' in kwargs:
            try:
                tmparg=str(kwargs['ylabel'])
                self.attrs['ylabel'] = tmparg
            except:
                raise Exception
        
        # Set title
        if 'title' in kwargs:
            try:
                tmparg = str(kwargs['title'])
                self.attrs['title'] = tmparg
            except: raise Exception
            
        if kwargs.keys() in 'xmin xmax ymin ymax zmin zmax'.split():
            plt.axis(**kwargs)
                                
        # If hold off delete previous lines
        if self.attrs['hold'] == 'off':
            self.lines = []
                    
        # self.lines.append(Line())
        if len(args) > 0: 
            self.parseargs(*args, **kwargs)
                    
    def parseargs(self, *args, **kwargs):
        """
        My own argument parser, supports easyplot and matlab style passing of
        arguments. It will create Line objects and append them to self.lines.
        """
        legendcounter=len(self.lines) # Used for telling how many lines were added
        # If first argument is a format string this will be ignored
        # If two format strings are used only the first of them will be used
        if 'x' in kwargs:   
            n = len(args)
            for i in range(len(args)-1):
                if not isinstance(args[i],str):
                    if isinstance(args[i+1],str):
                        self.lines.append(Line(x=kwargs['x'],
                                               y=args[i],
                                               format=args[1+i]))
                    else:
                        self.lines.append(Line(x=kwargs['x'],
                                               y=args[i],
                                               format=''))
                        if i == n-2:                
                            self.lines.append(Line(x=kwargs['x'],
                                                   y=args[i+1],
                                                   format=''))
        else: # Normal case
            # If an odd number, larger than 2, of non-strings in args are
            # between two string arguments, something is wrong.
            # If the odd number is one, the argument x='auto' is passed.
            n = len(args)
            i = 0
            if n in (1,2):
                if not isinstance(args[0], str):
                    if n == 1:
                        self.lines.append(Line(x='auto',
                                               y=args[0],
                                               format=''))
                    else:
                        if not isinstance(args[1],str):
                            self.lines.append(Line(x=args[0],
                                                   y=args[1],
                                                   format=''))
                        else:
                            self.lines.append(Line(x='auto',
                                                   y=args[0],
                                                   format=args[1]))
                    i+100 #return
                else:
                    raise Exception, "Can't plot a formatstring"
                
            while i <= (len(args)-3):
                # This item is not string --> y-value, should never be string.
                if not isinstance(args[i], str):
                    if not isinstance(args[i+1], str):
                        if isinstance(args[i+2], str):
                            self.lines.append(Line(x=args[i],
                                                   y=args[i+1],
                                                   format=args[i+2]))
                            i = i+3
                        else:
                            self.lines.append(Line(y=args[i+1],
                                                   x=args[i],
                                                   format=''))
                            i = i+2

                    # Next element is str --> no x-value
                    else:
                        self.lines.append(Line(y=args[i],
                                               x='auto',
                                               format=args[i+1]))
                        i = i+2
                    # These last cases could be run outside the while loop
                    if i == n-2: # Either y and format or x and y value left   
                        if isinstance(args[i+1], str):
                            self.lines.append(Line(y=args[i],
                                                   x='auto',
                                                   format=args[i+1]))
                        else:
                            self.lines.append(Line(x=args[i],
                                                   y=args[i+1],
                                                   format=''))
                    elif i == n-1:  # In this case we have only an y value left
                        self.lines.append(Line(y=args[i],
                                               x='auto',
                                               format=''))
    
        # Set legends
        if 'legend' in kwargs:
            # Number of new lines
            legendcounter = len(self.lines)-legendcounter
            legends=kwargs['legend']
            # If legends is a sequence
            if isinstance(legends, (tuple,list)):
                if len(legends) == legendcounter:
                    for i in range(legendcounter):
                        legend = legends[legendcounter-i-1]
                        if isinstance(legend,str):
                            self.lines[-1-i].attrs['legend'] = legend
                        else:
                            print "Legend "+legend+" is not a string"
                else:
                    print """Number of legend items is not equal to number \
                    of lines in plotcommand"""
            # Only one legend
            elif isinstance(legends,str):
                self.lines[-1].attrs['legend'] = legends
                
class BaseClass(object):
    """
    This base class saves info about plotting to instances of class Figure
    and Line. It also shows how the plot backend should behave. The plotbackend
    has to explicitly call Baseclass.func to use these features.
    
    If there is a two way communtication with the plot backend these features
    are not necessary, but the easy pickling to file is a good feature.
    """

    _formatchars = 'y m c r g b w k - : -. -- . o x + s ^ v'.split()
    # Dictionary of functions testing legal types
    _attrs_type = {'curfig': lambda arg: isinstance(arg, (int)),
                   'show':lambda arg: isinstance(arg, (bool)),
                   #'replot_mode':lambda arg: arg in ('Passive', 'Active'),
                   'interactive': lambda arg: isinstance(arg,(bool)),
                   #'changed': lambda arg: isinstance(arg, (bool)),
                   'hold': lambda arg: arg in ('on', 'off', None),
                   #'movie': lambda arg: arg in ('on', 'off', None),
                   'color': lambda arg: isinstance(arg,(bool))
                   }
        
    def __init__(self):
        BaseClass.init(self)

    def init(self):
        """Initialize internal data structures."""
        
        self._g = None  # Pointer to the backend for manual labour.
        self._figs = {1:Figure()} # Dictionary of figuresinstaces
        self._attrs = {
            'curfig':1,   # Current figure
            'show': True, # Screenplot after each plot command
            #'replot_mode':'Active', # Update backend after each change
            #'changed':False,        # sync state       
            'interactive': False,
            'hold': None,
            #'movie':None,            # Axis are locked
            'color':False            # Hardcopy with color?
            }
        
    def set(self, **kwargs):
        for key in kwargs:
            value = kwargs[key]
            if key in self._attrs:  # legal key?
                if self._attrs_type[key](value):  # legal type?
                    self._attrs[key] = value
                else:
                    raise TypeError, \
                          'BaseClass.set: keyword "%s" %s is illegal.' % \
                          (key, type(key))
            
    def get(self, prm_name):
        if prm_name in self._attrs:
            return self._attrs[prm_name]
        else:
            KeyError, 'Figure.get: no parameter with name "%s"' % prm_name
        
    def _replot(self, forceupdate=True, *args, **kwargs):
        """
        Method to update backend after change in data.
        This method must be written in backend.
        """
        raise AttributeError, '_replot not implemented in class %s' % \
              self.__class__.__name__
    
    def openfig(self, filename='figspickle.txt'):
        """Opens a figure saved in internal format."""

        handle = open('trill.pickle', 'r')
        self._figs
        tmp = pickle.load(handle)
        handle.close()
        
        fail = True
        if isinstance(tmp, dict):
            fail = False
            for item in tmp:
                if not isinstance(item, Figure):
                    fail = True
        if fail:
            raise Exception ("Import error. Cannot retrieve figures \
            from filename %s ." %filename)
          
        self._figs.update(tmp)  
                                         
    def savefig(self, filename='figspickle.txt'):
        """Save a figure in internal format."""
        # Actually saves all figures

        handle = open(filename, 'w')
        pickle.dump(self._figs, handle)
        handle.close()
    
    def hardcopy(self, filename='tmp.eps', *args):
        """
        Usage: hardcopy(filename) or hardcopy()
        Will save the Figure to file named filename. 
        The format of the file is determined from the extension of filename.
        If format is not supported  PostScript output will be used.
        """
        raise AttributeError, 'hardcopy not implemented in class %s' % \
              self.__class__.__name__
    
    def hold(self, arg='on'):
        """ Change the hold state of current figure."""
        # Is this the way it should be?
        # Should hold state be global for backend?
        
        if not arg in ('on', 'off'):
            if arg or arg == None:
                arg = 'on'
            else:
                arg = 'off'
        self._figs[self._attrs['curfig']].attrs['hold'] = arg            
        
    def figure(self, num=None):
        """
        Create a new figure or switch between figures.
        num is the figure number of the new or existing figure.
        """
        try: num = int(num)
        except:
            # print str(num),' is not an integer'
            if len(self._figs) == 0: #No figures left
                num = 1
            else:
                num=max(self._figs.keys())+1
                print "Active figure is %g."%num

        if not num in self._figs:
            # Points to class Figure or other convenient function
            # In gnuplot backend this should instanciate a new pipe instead
            self._figs[num] = Figure()             
                                              
        self._attrs['curfig'] = num
        
    def clf(self):
        """Clear figure."""    
        del self._figs[self.get('curfig')]
        self.figure(self.get('curfig'))
        
    def axis(self, *args, **kwargs):
        """
        Choose the  axis limits.
        axis([xmin, xmax, ymin, ymax [, zmin, zmax]])
        axis('auto'): autoscaling will be used
        """
        fig = self._figs[self._attrs['curfig']]    
        keys = ['xmin', 'xmax', 'ymin', 'ymax', 'zmin', 'zmax']
        
        # Allow both of axis(xmin,xmax,ymin,ymax) and axis([xmin,...])
        if (len(args)==1):
            if isinstance(args[0],(tuple,list)):
                args=args[0]
            elif isinstance(args[0],str):
                if args[0] == 'auto':
                    for attr in keys:
                        fig.attrs[attr] = None
                return
        # first treat positional arguments:
        for i in range(len(args)):
            fig.attrs[keys[i]] = args[i]
        # allow for keyword arguments:
        for kw in keys:
            if kw in kwargs:
                fig.attrs[kw] = kwargs[kw]
                
        if self.get('interactive') and self.get('show'):
            self._replot()
            
    def closefig(self, arg):
        """Close figure window."""
        raise AttributeError, 'closefig not implemented in class %s' % \
              self.__class__.__name__
        
    def closefigs(): 
        """Close all figure windows."""
        self._figs = {}
        self._figs[1] = Figure()
        self._attrs['curfig'] = 1
        #The rest should be written in subclass
                
    def grid():
        """Toggle the display of grid."""
        raise AttributeError, 'grid not implemented in class %s' % \
              self.__class__.__name__
    
    '''def box():
        """ Toggle whether graph is boxed."""
        pass            
    '''
    
    '''#Only level 1 conformance
    def multi(self):
        """Split a figure into a ``MxN`` grid"""
        pass
    
    def refocus(self):
        """Focus on a subplot created by multi"""
        pass
    
    def subplot(self): 
        """Create multiple plots in on figure. **DEPRECATED**: use `multi`"""
        pass
    '''
    
    def legend(self, *args):
        """Add a legend to a graph."""

        # Write out short name
        lines = self._figs[self._attrs['curfig']].lines
        if len(args)>1:
            # Consistency check of len(args) and number of lines in fig
            if len(lines) == len(args):
                # Iterate over lines and set legend
                for i in range(len(args)):
                    lines[i].attrs['legend'] = str(args[i])
        elif len(args) == 1:
            lines[-1].attrs['legend'] = str(args[0])

        if self.get('interactive'):
            self._replot()

    def title(self, label):
        """Title a graph."""
        self._figs[self._attrs['curfig']].attrs['title'] = str(label)
        if self.get('interactive'):
            self._replot()
        
    def xlabel(self, label): 
        """Label the x-axis."""
        self._figs[self._attrs['curfig']].attrs['xlabel'] = str(label)
        if self.get('interactive'):
            self._replot()
        
    def ylabel(self, label):
        """ Label the y-axis."""
        self._figs[self._attrs['curfig']].attrs['ylabel'] = str(label)
        if self.get('interactive'):
            self._replot()
        
    # 2D Plotting
    def plot(self, *args, **kwargs):
        """
         Draw line and scatter plots. 
 
            - ``plot(y)``
                plots values in `y` on y-axis (same as 
                ``plot(range(len(y)),y)`` )  
 
            - ``plot(x, y)``
                plots `y` against `x`, i.e. if `x` and `y` are vectors of
                length n, then this will plot all the points (x[i], y[i]) for
                0<=i<n. 
 
            - ``plot(y, fmt)``
                plots values in `y` on y-axis formated like `fmt` (see below).
 
            - ``plot(x1,y1,fmt1, x2,y2,fmt2, ...)``
                    same as ``hold(1)`` followed by multiple ``plot(x,y,fmt)``.
 
            - ``plot(x1,y1,x2,y2,...)``
                like above, but automatically chooses different colours.
                
            - ``plot(y1,...,x=x)``
                uses `x` as the x values for all the supplied
                curves. ``x='auto'`` has the same effect as
                ``x=range(len(y1))``.
 
            - ``plot(...,log='x')``, ``plot(...,log='xy')`` etc. draw
                logarithmic axes
            
 
        the following format specifiers exist ::
 
            y     yellow        .     point              -     solid
            m     magenta       o     circle             :     dotted
            c     cyan          x     x-mark             -.    dashdot 
            r     red           +     plus               --    dashed   
            g     green         *     star
            b     blue          s     square
            w     white         d     diamond
            k     black         v     triangle (down)
                                ^     triangle (up)
                                <     triangle (left)
                                >     triangle (right)
                                p     pentagram
                                h     hexagram
 
        Examples:
        
        draws a line
        >>> plot([1,2,3])
 
        draws 3 red 'x'
        >>> plot([1,2,3], 'rx')
 
        Note: `loglog`, `semilogy`, `semilogx` are like ``plot(...,log='xy')``,
        ``plot(...,log='y')`` and ``plot(...,log='x')``.
        """     
        self._figs[self.get('curfig')].set(*args,**kwargs) 
                
        if 'color' in kwargs:
            self.set(color=kwargs['color'])
        if 'show' in kwargs:
            self.set(show=kwargs['show'])
        if 'interactive' in kwargs:
            self.set(interactive=kwargs['interactive'])

        if 'axis' in kwargs:
            self.axis(*kwargs['axis'])
            
        if 'hardcopy' in kwargs:
            self.hardcopy(kwargs['hardcopy'])
            
        if (self.get('interactive') and self.get('show')) or self.get('show'):
            self._replot()

    def show(self):
        self._replot()

    def loglog(self, *args, **kwargs):
        """ Like `plot`: with logarithmic x and y axes
        (same as ``plot(..., log='xy')``)."""
        kwargs['log'] = 'xy'
        self.plot(*args, **kwargs)
        
    def semilogx(self, *args, **kwargs): 
        """ Like `plot`: with logarithmic x axis.
        (same as ``plot(..., log='x')``)."""
        kwargs['log'] = 'x'
        self.plot(*args, **kwargs)
        
    def semilogy(self, *args, **kwargs): 
        """Like `plot`: with logarithmic y axis.
        (same as ``plot(..., log = 'y')``)."""
        kwargs['log'] = 'y'
        self.plot(*args, **kwargs)

        
class DerivedClass(BaseClass):
    """This is a template for creating new backends"""
    
    def __init__(self):
        BaseClass.__init__(self)
        self.init()
        
    def init(self):
        # Set docstrings of all functions to the docstrings of BaseClass
        # The exception is if something is very different
        # Alex had a nice function for this.

        #Do initialisation special for this backend
        pass
        
    def _replot(self):
        """
        Update backend
        """
        #Write backend specific plot commands
        '''
        currentfignumber=self._attr('curfig')
        currentfig=self._figs{currentfignumber}
        1. set correct attributes in backend

        for line in currentfig.lines:
            2 check attributes of line
            3 plot data.
        
        '''
        pass
        
    def hardcopy(self, file='file.ps', **kwargs):
        """
        Save current figure to file
        """
        pass
    
    #def somefunc(self,*args,**kwargs):
    #    BaseClass.somefunc(self,*args,**kwargs)
    #    #Add extra functionality here

    
# Use /setup config file     
# filestring is default setup
# NOT FINISHED!
filestring = """
_backends=('blt_', 'pyx_', 'gnuplot_')

default = 'gnuplot' 
_attrs = {}
"""

"""
try:
    inputfile = os.environ['HOME']+'/.rolvplotrc.py'
    if not os.path.isfile(inputfile):
        file = open(inputfile,'w')
        file.write(filestring)
        file.close()
    execfile(inputfile)

except:
    exec(filestring)
"""
#def use:
#    for item in f:
#        eval('global %s' % item)
#        exec '%s = plt.%s' % (item,item)
#        __all__.append(item)

def use(plt, namespace=globals()):
    """
    Export the namespace of backend instance to namespace.
    """
    plt_dict = {}
    plt_dict['plt'] = plt
    for item in plt.__dict__:
        plt_dict[item] = eval('plt.'+item)                                   
    for item in dir(plt.__class__):
        if not '__' in item:  
            plt_dict[item] = eval('plt.'+item) 
    namespace.update(plt_dict)  # Add to global namespace 

    # If this module is imported
    try:
        __all__
    except:
        __all__ = ['plt']
    try:
        for item in plt_dict.keys():
            __all__.append(item)
    except:
        pass
    del(__all__)

def debug(plt):

    def prettyprint(d):
	for key in d:
	    print key,"\t: ", d[key]
	print "\n"
	
    print "plt._attrs: " 
    prettyprint(plt._attrs)
    for fignr in plt._figs:
	print "Fig: ",fignr
	print "fig._attrs: " 
	prettyprint(plt._figs[fignr].attrs)
	
	for line in plt._figs[fignr].lines:
	    print line
	    prettyprint(line.attrs)
