"""Tests of  the easyplot package"""

import unittest
import glob
import random
from Numeric import *
from time import sleep

try:
    from py4cs.numpytools import *
except:
    seq = arange # Not the same, but close

_plts = {}
from easyplot.gnuplot_ import *
_plts['gnuplot'] = plt
"""try:
    from easyplot.pyx_ import *
    _plts['pyx_'] = plt
except:
    pass
"""
try:
    from easyplot.blt_ import *
    _plts['blt'] = plt
except:
    pass


class bltTest(unittest.TestCase):
    """Blt specific tests"""
    
    def setUp(self):
        self.x1 = arange(0, 1, 0.01)
        self.y1 = sin(2*pi*self.x1)
        self.x2 = arange(0, 2, 0.01)
        self.y2 = cos(2*pi*self.x2)
        self.time = 1.0
        try:
            use(_plts['blt'], globals())
            set(interactive=True,
                color=True) 
        except:
            use(_plts['gnuplot'],globals())
            return # Hopefully will no testcases of this class run...
        
    def test(self):
        self.setUp()
        try:
            figure()
            plot((1,2,3), (4,6,5), 'ro--')
            legend('tril')
            axis(0, 5, 0, 10)
            hold('on')
            plot((2,4,5), (4,4,4), '')
            legend('notrill', 'trill2')
            plot((1,2,3,4,5,6), (1,3,2,4,3,6), 'k:x')
            legend('nils')
            import Numeric
            x=Numeric.array((1,2,3,4))
            plot(x, x**2, 'y')
            axis(0, 10, 0, 10)
            hardcopy('blt_test1.ps')
            figure()
            plot((1,2,3), 'r:', (2,3,4), 'b-', (3,4,5), 'k-.',\
                 (4,6,6), 'g--')#, x='auto')
            legend('dotted', 'solid', 'dotdashed', 'dashed')
            hardcopy('blt_test2.ps')    
            figure()
            plot((1,10,100), log='y')
            hold('on')
            loglog((1,19,199), (10,100,1000))
            legend('loglog', 'loglog')
            title('one scale')
            xlabel('x')
            ylabel('y')
            hardcopy('blt_test3.ps')
        except:
            print "Error: running debug(plt)"
            debug(plt)
            
class gnuplotTest(unittest.TestCase):
    """Gnuplot specific tests"""

    def setUp(self):
        self.x1 = arange(0, 1, 0.01)
        self.y1 = sin(2*pi*self.x1)
        self.x2 = arange(0, 2, 0.01)
        self.y2 = cos(2*pi*self.x2)
        self.time = 1.0
        use(_plts['gnuplot'],globals())
        set(interactive=False,
            color=True)
        set(show=False)
    #print 'gnuplotTest'

    def test1(self):
        #print 'test1'
        clf()
        plot((1,2,3), (4,5,4), 'bo-')
        
        #plt.legend('1')
        hold('on')
        plt.plot((1, 2, 3), 'ko-', (3, 1, 1), 'r.-', x=(1, 2, 3))
        axis(0, 6, 0, 6)
        legend('1', '2', '3')
        title('y(x)')
        xlabel('x')
        ylabel('y')
        filename = 'gnuplottest1.eps' # Either ps or eps.
        plot((1,2,3), 'r:', (2,3,4), 'b-', (3,4,5), 'k-.', (4,6,6), 'g--', \
             x='auto')
        hardcopy(filename, color=True)
        
        
        
    def test2(self):
        #print 'test2'
        set(color=True)
        clf()
        #print plt._figs
        #print get('curfig')
        
        plot((1,2,3), '.', (2,3,4), 'ro--',  xlabel='xlabel',  \
             ylabel='ylabel', title='title', legend=('1','2'))
        x = arange(0, 10, 0.1)
        y1 = cos(x)
        y2 = sin(x)
        axis([0, 3, -1, 6])
        plot(y1, 'r.', y2, 'bo', x=x, legend=('cos(x)','sin(x)'), color=True,
             show=False, hardcopy='gnuplottest2.ps',)
        
        plt._g('quit')
        #show()
        
        
class pyxTest(unittest.TestCase):
    """Pyx specifig test"""
    def test1(self):
        try:
            if  'pyx_' in _plts:
                use(_plts['pyx_'], globals())
                figure()
                plot((2, 2, 3, 4), 'kv-', (3, 3, 4, 6), 'kv', x='auto')
                axis(-1, 5, -1, 10)
                title('A very long title')
                legend('line 1', 'line 2')
                #latextext = r"$2\pi\gamma k\Omega$"
                #$plt._g.text(0, 0, latextext)
                hold('on')
                plot((5, 3, 5, 5))
                legend('line 3')
                hardcopy('testpyx.eps')
        except:
            print 'pyx_ error: \nDebugging plt'
            #debug(plt)

        
class easyplotTest(unittest.TestCase):
    """These tests are run for all implemented backends"""

    def whattodointest(self):
        print "Now testing %s" %plt.__module__.split('.')[1]
        
        
        #_colors = "b g r m c y k".split()
        #_markers = "o + x * s d v ^ < > p h .".split()
        #_linestyles = "- : -. --".split()
        _colors = Line._colors
        _markers = Line._markers
        _linestyles = Line._linestyles
        #_markers.append('') # No Marker option
        #_linestyles.append('') # No Linestyle option
        # Note if no marker or no linestyle, default '-x' is used

        if plt.__module__.split('.')[1] in ('pyx_', 'blt'):
            _markers = ['v','+','s','x','^','o'] 
      
        x = seq(0, 2, 0.2)
        y = sin(x)*x
        print _linestyles
        
        # Generate formatstrings
        format = []
        for item in _markers:
            format.append(''.join([
                random.choice(_colors),
                item,
                random.choice(_linestyles)
        ]))
        figure()
        hold('on')
        set(color=True)
        set(show=False)
        set(interactive=False)
        for i in range(len(format)):
            y = sin(x+i)*x
            fmt = format[i]
            plot(x, y, fmt, legend=fmt)
        #legend(fmt)
        #print plt._figs[1].lines[i].attrs['legend']
        #legend(*format)
        plot((1,2,3), (2,3,4), x='auto', legend=('nestsist', 'sist'))
        #axis([0,2,0,5]) # Prefered way of using the axis command
        axis(min(x), max(x)+1, -2, 5) # Note there is no tuple here
        
        hardcopy('%s_style_test.ps' %plt.__module__.split('.')[1])
        clf()
        #debug(plt)
        
    def test1(self):
        for plt in _plts:
            if not plt == 'pyx_': #Something is very bad with pyx
                use(_plts[plt],globals())
                self.whattodointest()
        
        
#suite = unittest.makeSuite(EasyplotTest, 'test')
#suite = unittest.makeSuite(bltTest, 'test')
#suite = unittest.makeSuite(gnuplotTest, 'test')

def main():    
    postscriptfiles = '' + \
                      " ".join(glob.glob('*.eps')) + \
                      " ".join(glob.glob('*.ps'))
    if len(postscriptfiles) > 0:
        print "Warning: you have old ps/eps files in testdir."
        print "These files might be overwritten by this test"
        print "The files are: ",postscriptfiles    
        #rm *.ps *.eps *.pyc *~ -f

    #unittest.main() # There is a sys.exit() inside here....
    os.system("python -c '''from easyplot.unittest_ import *;unittest.main()'''")
    
    if os.uname()[0] == 'Linux':
	psviewer = 'gv'
	for file in glob.glob('*.ps'):
            os.system('%s %s &' %(psviewer, file))
        for file in glob.glob('*.eps'):
            os.system('%s %s &' %(psviewer, file))
        #raw_input('Press return when files are examined')
    else:
        show()
       
if __name__ == "__main__":
    main()
