"""
Visualization of fields over grids.
"""

from BoxGrid import X, Y, Z

def gnuplot(grid, data, g=None, legend='', plane_in_3D=(), ps=False):
    """
    Visualize 1D/2D/3D scalar field with Gnuplot.
    data:  array with field values at the grid points
    """
    import Gnuplot
    if g is not None:
        if not isinstance(g, Gnuplot.Gnuplot):
            raise TypeError, \
                  'gnuplot(g): g must be a Gnuplot instance, '\
                  'not a %s' % type(g)
        external_g = True  # external Gnuplot object
    else:
        g = Gnuplot.Gnuplot(persist=1)
        external_g = False # internally created Gnuplot object

    try:
        grid.compatible(data)
    except (IndexError,TypeError), e:
        raise e.__class__, 'BoxGrid.gnuplot:\n%s' % e

    if grid.nsd == 1:
        # curve plot:
        d = Gnuplot.Data(grid.coor[0], data, with='lines', title=legend)
        g.plot(d)
    elif grid.nsd == 2:
        # surface plot:
        g('set pm3d')
        g('set palette')
        g('unset hidden')
        g('unset surface')
        if grid.nx < 30 and grid.ny < 30:
            # plot grid lines if they are not too dense
            g('set style line 100 lt 5 lw 0.5')
            g('set pm3d solid hidden3d 100')
        else:
            g('set pm3d at s')
        g('set view 50,220')
        g('set xrange [%g:%g]' % (grid.xmin, grid.xmax))
        g('set yrange [%g:%g]' % (grid.ymin, grid.ymax))
        g('set isosamples 30')
        g('set title "%s"' % legend)
        g.splot(Gnuplot.GridData(data, grid.coor[0], grid.coor[1]))
    elif grid.nsd == 3:
        if not plane_in_3D:
            print 'BoxGrid.gnuplot: cannot plot 3D field; fix a plane,\n'\
                  'e.g. grid.gnuplot(data, plane_in_3D=(X,5)) plots\n'\
                  'the 3D data array for a plane x=g.coor[X][5]'
        else:
            # extract data:
            fixed_coor, index = plane_in_3D
            if fixed_coor == X:
                slice_2D = data[index,:,:]
                x, y = grid.coor[1:3]
                g('set xlabel "y"')
                g('set ylabel "z"')
            elif fixed_coor == Y:
                slice_2D = data[:,index,:]
                x, y = grid.coor[0], grid.coor[2]
                g('set xlabel "x"')
                g('set ylabel "z"')
            elif fixed_coor == Z:
                slice_2D = data[:,:,index]
                x, y = grid.coor[0:2]
                g('set xlabel "x"')
                g('set ylabel "y"')
            d = Gnuplot.GridData(slice_2D, x, y)
            g('set pm3d')
            g('set palette')
            g('unset hidden')
            g('unset surface')
            g('set title "%s"' % legend)
            if len(x) < 30 and len(y) < 30:
                # plot grid lines if they are not too dense
                g('set style line 100 lt 5 lw 0.5')
                g('set pm3d solid hidden3d 100')
            else:
                g('set pm3d at s')
            g('set view 50,220')
            g('set xrange [%g:%g]' % (x[0], x[-1]))
            g('set yrange [%g:%g]' % (y[0], y[-1]))
            g('set isosamples 30')
            g.splot(d)

    if ps:
        if isinstance(ps, basestring):
            filename = ps
        else:
            filename = 'tmp.ps'
        g.hardcopy(filename, color=True)

    if not external_g:
        # give g some time to plot before g is destroyed at return...
        import time
        time.sleep(3)

# Vector fields, see vector.dem plot in gnuplot-4.0/demo

